import React, { PropTypes } from 'react'
import {
  SpaceScale,
  SpaceScalePropType,
  NegativeIncludedSpaceScalePropType,
} from './../components/Space'

const SpacingPropType = PropTypes.oneOfType([
  PropTypes.bool,
  SpaceScalePropType
])

const NegativeIncludedSpacingPropType = PropTypes.oneOfType([
  PropTypes.bool,
  NegativeIncludedSpaceScalePropType
])

export const propTypes = {
  padding: SpacingPropType,
  paddingTop: SpacingPropType,
  paddingRight: SpacingPropType,
  paddingBottom: SpacingPropType,
  paddingLeft: SpacingPropType,
  paddingX: SpacingPropType,
  paddingY: SpacingPropType,
  margin: NegativeIncludedSpacingPropType,
  marginTop: NegativeIncludedSpacingPropType,
  marginRight: NegativeIncludedSpacingPropType,
  marginBottom: NegativeIncludedSpacingPropType,
  marginLeft: NegativeIncludedSpacingPropType,
  marginX: NegativeIncludedSpacingPropType,
  marginY: NegativeIncludedSpacingPropType
}

const implicitSpacingSystemWarning = (key, val) => {
  console.group('react-ui-library')
  console.warn(`Please update the use of '${key}={${val}}' to a string value padding='__px' \n The implicit spacing system will be deprecated in v1.0.0`, 'https://github.com/segmentio/react-ui-library/issues/38')
  console.groupEnd()
}

const testIfImplicitSpacing = props => {
  Object.keys(props).forEach(key => {
    if (props[key] !== undefined && typeof props[key] !== 'string') {
      implicitSpacingSystemWarning(key, props[key])
    }
  })
}

const parseUnit = (unit) => {
  if (typeof unit === 'boolean') {
    return SpaceScale[4]
  } else if (Number.isInteger(unit)) {
    if (unit < 0) {
      return `-${SpaceScale[Math.abs(unit)]}`
    }
    return SpaceScale[unit]
  }
  return unit
}

const parseStyleProps = ({ all, top, right, bottom, left }) => {
  const pAll = parseUnit(all) || 0
  return `${parseUnit(top) || pAll} ${parseUnit(right) || pAll} ${parseUnit(bottom) || pAll} ${parseUnit(left) || pAll}`
}

export const parseProps = ({
  padding,
  paddingTop,
  paddingRight,
  paddingBottom,
  paddingLeft,
  paddingX,
  paddingY,
  margin,
  marginTop,
  marginRight,
  marginBottom,
  marginLeft,
  marginX,
  marginY,
  style,
  ...props
}) => {
  const styles = {}

  testIfImplicitSpacing({
    padding: padding,
    paddingTop: paddingTop,
    paddingRight: paddingRight,
    paddingBottom: paddingBottom,
    paddingLeft: paddingLeft,
    paddingX: paddingX,
    paddingY: paddingY,
    margin: margin,
    marginTop: marginTop,
    marginRight: marginRight,
    marginBottom: marginBottom,
    marginLeft: marginLeft,
    marginX: marginX,
    marginY: marginY })

  if (padding !== undefined
  || paddingTop !== undefined
  || paddingRight !== undefined
  || paddingBottom !== undefined
  || paddingLeft !== undefined
  || paddingX !== undefined
  || paddingY !== undefined) {
    styles.padding = parseStyleProps({
      all: padding,
      top: paddingTop !== undefined ? paddingTop : paddingY,
      right: paddingRight !== undefined ? paddingRight : paddingX,
      bottom: paddingBottom !== undefined ? paddingBottom : paddingY,
      left: paddingLeft !== undefined ? paddingLeft : paddingX
    })
  }

  if (margin !== undefined
  || marginTop !== undefined
  || marginRight !== undefined
  || marginBottom !== undefined
  || marginLeft !== undefined
  || marginX !== undefined
  || marginY !== undefined) {
    styles.margin = parseStyleProps({
      all: margin,
      top: marginTop !== undefined ? marginTop : marginY,
      right: marginRight !== undefined ? marginRight : marginX,
      bottom: marginBottom !== undefined ? marginBottom : marginY,
      left: marginLeft !== undefined ? marginLeft : marginX
    })
  }

  return { style: { ...styles, ...style }, ...props }
}
